@@ -79,7 +79,7 @@ ZMRIMULTISegmentation::ZMRIMULTISegmentation(){}
 
 const float PI=3.14159265;
 
-void ZMRIMULTISegmentation::TanakaCreate(const NEWIMAGE::volume<float>* images,  int nclasses, bool b2d, int nbiter,float nblowpass, float fbeta, int bapused, bool pveboolean, int nchan, int altype, bool bbias, int initinitfixed, bool verb, int biterationspve, int winit, float fpveMRFmixeltyp, float Hyp,string mansegfle, int typeoffile)
+void ZMRIMULTISegmentation::TanakaCreate(const NEWIMAGE::volume<float>* images,  int nclasses, bool b2d, int nbiter,float nblowpass, float fbeta, int bapused, bool pveboolean, int nchan, bool bbias, int initinitfixed, bool verb, int biterationspve, int winit, float fpveMRFmixeltyp, float Hyp,string mansegfle, int typeoffile)
 {
   mansegfile=mansegfle; 
   verboseusage=verb;
@@ -316,9 +316,9 @@ void ZMRIMULTISegmentation::pveClassification(int x, int y, int z)
 
 void ZMRIMULTISegmentation::pveClassification()
 {
-   for(int x=1;x<m_nWidth;x++)
-     for(int y=1;y<m_nHeight;y++)
-       for(int z=1;z<m_nDepth;z++)
+   for(int x=0;x<m_nWidth;x++)
+     for(int y=0;y<m_nHeight;y++)
+       for(int z=0;z<m_nDepth;z++)
 	 pveClassification(x, y, z);
 }
 
@@ -543,11 +543,11 @@ void ZMRIMULTISegmentation::PVClassificationStep()
   Matrix invsig(numberofchannels, numberofchannels);
   if(noclasses==3)
     {
-      for(int z=1;z<m_nDepth-1;z++)
+      for(int z=0;z<m_nDepth;z++)
 	{
-	  for(int y=1;y<m_nHeight-1;y++)
+	  for(int y=0;y<m_nHeight;y++)
 	    {
-	      for(int x=1;x<m_nWidth-1;x++)
+	      for(int x=0;x<m_nWidth;x++)
 		{ 
 		  if(m_mask.value(x, y, z)==1)
 		    {
@@ -620,11 +620,11 @@ void ZMRIMULTISegmentation::PVClassificationStep()
 		}
 	      float detsig=sig.Determinant();
 	      invsig=sig.i();
-	      for(int z=1;z<m_nDepth-1;z++)
+	      for(int z=0;z<m_nDepth;z++)
 		{
-		  for(int y=1;y<m_nHeight-1;y++)
+		  for(int y=0;y<m_nHeight;y++)
 		    {
-		      for(int x=1;x<m_nWidth-1;x++)
+		      for(int x=0;x<m_nWidth;x++)
 			{ 
 			  if(m_mask.value(x, y, z)==1)
 			    {
@@ -638,11 +638,11 @@ void ZMRIMULTISegmentation::PVClassificationStep()
     }
   if(noclasses>3)
     {
-      for(int z=1;z<m_nDepth-1;z++)
+      for(int z=0;z<m_nDepth;z++)
 	{
-	  for(int y=1;y<m_nHeight-1;y++)
+	  for(int y=0;y<m_nHeight;y++)
 	    {
-	      for(int x=1;x<m_nWidth-1;x++)
+	      for(int x=0;x<m_nWidth;x++)
 		{ 
 		  if(m_mask.value(x, y, z)==1)
 		    {
@@ -685,11 +685,11 @@ void ZMRIMULTISegmentation::PVClassificationStep()
 		}
 	      float detsig=sig.Determinant();
 	      invsig=sig.i();
-	      for(int z=1;z<m_nDepth-1;z++)
+	      for(int z=0;z<m_nDepth;z++)
 		{
-		  for(int y=1;y<m_nHeight-1;y++)
+		  for(int y=0;y<m_nHeight;y++)
 		    {
-		      for(int x=1;x<m_nWidth-1;x++)
+		      for(int x=0;x<m_nWidth;x++)
 			{ 
 			  if(m_mask.value(x, y, z)==1)
 			    {
@@ -711,11 +711,11 @@ void ZMRIMULTISegmentation::PVClassificationStep()
 		mu(i, 1)=m_mean(type+1, i);
 	      invsig=m_co_variance[type+1].i();
 	      float detsig=m_co_variance[type+1].Determinant();
-	      for(int z=1;z<m_nDepth-1;z++)
+	      for(int z=0;z<m_nDepth;z++)
 		{
-		  for(int y=1;y<m_nHeight-1;y++)
+		  for(int y=0;y<m_nHeight;y++)
 		    {
-		      for(int x=1;x<m_nWidth-1;x++)
+		      for(int x=0;x<m_nWidth;x++)
 			{ 
 			  if(m_mask.value(x, y, z)==1)
 			    {
@@ -742,11 +742,11 @@ void ZMRIMULTISegmentation::PVClassificationStep()
 		      }
 		    float detsig=sig.Determinant();
 		    invsig=sig.i();
-		    for(int z=1;z<m_nDepth-1;z++)
+		    for(int z=0;z<m_nDepth;z++)
 		      {
-			for(int y=1;y<m_nHeight-1;y++)
+			for(int y=0;y<m_nHeight;y++)
 			  {
-			    for(int x=1;x<m_nWidth-1;x++)
+			    for(int x=0;x<m_nWidth;x++)
 			      { 
 				if(m_mask.value(x, y, z)==1)
 				  {
@@ -789,11 +789,11 @@ void ZMRIMULTISegmentation::ICMPV()
   if(noclasses>3)
     mixnum=2*noclasses-1;
   float* clique=new float[mixnum];
-  for(int z=1;z<m_nDepth-1;z++)
+  for(int z=0;z<m_nDepth;z++)
     {
-      for(int y=1;y<m_nHeight-1;y++)
+      for(int y=0;y<m_nHeight;y++)
 	{
-	  for(int x=1;x<m_nWidth-1;x++)
+	  for(int x=0;x<m_nWidth;x++)
 	    { 
 	      if(m_mask.value(x, y, z)==1)
 		    {
@@ -812,11 +812,11 @@ void ZMRIMULTISegmentation::ICMPV()
     }
   for(int iter=0;iter<1;iter++)
     {
-      for(int z=1;z<m_nDepth-1;z++)
+      for(int z=0;z<m_nDepth;z++)
 	{
-	  for(int y=1;y<m_nHeight-1;y++)
+	  for(int y=0;y<m_nHeight;y++)
 	    {
-	      for(int x=1;x<m_nWidth-1;x++)
+	      for(int x=0;x<m_nWidth;x++)
 		{ 
 		  if(m_mask.value(x, y, z)==1)
 		    {
@@ -973,11 +973,11 @@ void ZMRIMULTISegmentation::PVestimation()
   float step=(float)(1.0f/(float)(iterationspve));
   Matrix mu(numberofchannels, 1);
   Matrix sig(numberofchannels, numberofchannels);  
-  for(int z=1;z<m_nDepth-1;z++)
+  for(int z=0;z<m_nDepth;z++)
     {
-      for(int y=1;y<m_nHeight-1;y++)
+      for(int y=0;y<m_nHeight;y++)
 	{
-	  for(int x=1;x<m_nWidth-1;x++)
+	  for(int x=0;x<m_nWidth;x++)
 	    { 
 	     if(m_mask.value(x, y, z)>0)
 	       {
@@ -1259,6 +1259,7 @@ void ZMRIMULTISegmentation::InitSimple(const NEWIMAGE::volume<float>& pcsf, cons
 	    }
 	}
     }
+  UpdateMembers(m_post);
   MeansVariances(noclasses);
 }
 
@@ -1315,7 +1316,7 @@ void ZMRIMULTISegmentation::MeansVariances(int numberofclasses)
   float* normtemp=new float[noclasses+1];
   for(int c=1;c<noclasses+1;c++)
     normtemp[c]=0.0f;
-  for(int c=1;c<numberofchannels+1;c++)
+  for(int c=1;c<noclasses+1;c++)
     {
       m_co_variance[c]=0.0f;
       for(int i=1;i<=numberofchannels;i++)
@@ -1536,34 +1537,26 @@ void ZMRIMULTISegmentation::UpdateWeights()
 
 void ZMRIMULTISegmentation::WeightedKMeans()
 {
-
-
-ifstream inputfile;
-  float* input_mean = new float[noclasses];
-  int input_c=0;
+  vector<float> inputMeans;
   if (mansegfile!="") 
   {  
-    inputfile.open(mansegfile.c_str(), ifstream::in);
-    while ( (inputfile >> input_mean[++input_c]) && (input_c<=(noclasses*numberofchannels)) );
+    ifstream inputfile(mansegfile.c_str());
+    copy(istream_iterator<float> (inputfile),istream_iterator<float> (),back_inserter(inputMeans));
     inputfile.close();
   }
 
   m_post=m_prob=0.0f;
   for(int z=0;z<m_nDepth;z++)
-    {
-      for(int y=0;y<m_nHeight;y++)
-	{
-	  for(int x=0;x<m_nWidth;x++)
-	    {
-	      if(m_mask(x, y, z)==1)
-		{
-		  m_maskc(x, y, z)=1.0f;
-		}
-	      else
-		  m_maskc(x, y, z)=0.00f;
-	    }
-	}
-    }
+    for(int y=0;y<m_nHeight;y++)
+      for(int x=0;x<m_nWidth;x++)
+      {
+	 if(m_mask(x, y, z)==1)
+	   m_maskc(x, y, z)=1.0f;
+	 else
+	   m_maskc(x, y, z)=0.00f;
+      }
+	
+    
   m_mean.ReSize(noclasses+1, numberofchannels);m_mean=0.0;
   m_co_variance=new Matrix[noclasses+1];
   m_inv_co_variance=new Matrix[noclasses+1];
@@ -1580,7 +1573,7 @@ ifstream inputfile;
       float perc=1.0/((float)(noclasses+1.0));
       for(int c=1;c<noclasses+1;c++)
 	{
-          if ( input_c == ((noclasses*numberofchannels)+1) ) m_mean(c, n)=log(input_mean[c+noclasses*(n-1)]); 
+          if ( (int)inputMeans.size() == (noclasses*numberofchannels) ) m_mean(c, n)=log(inputMeans[c+noclasses*(n-1)-1]); 
 	  else m_mean(c, n)=m_Mricopy[n].percentile((float)(perc*c), m_maskc);
            if (verboseusage) cout << n << " " << c << " " << m_mean(c, n) << endl ;
 	}
@@ -1630,7 +1623,6 @@ ifstream inputfile;
 	cout<<"KMeans Iteration "<<initfiter<<"\n";
       m_post=m_prob=Initclass(noclasses);
     }
-  delete [] input_mean;
 }
 
 int ZMRIMULTISegmentation::qsort()
@@ -1642,7 +1634,7 @@ int ZMRIMULTISegmentation::qsort()
     Matrix* tempvar=new Matrix[noclasses+1];
     Matrix* invvarcopy=new Matrix[noclasses+1];
     Matrix* tempinvvar=new Matrix[noclasses+1];
-    if(imagetype==1) // for a T2 image reverse the intensity order of _master image_
+    if(imagetype==2) // for a T2 image reverse the intensity order of _master image_
       for(c=1; c<noclasses+1; c++)
 	m_mean(c,1)*=-1.0f;
     for(c=1; c<noclasses+1; c++)
@@ -1682,7 +1674,7 @@ int ZMRIMULTISegmentation::qsort()
         m_inv_co_variance[c]=tempinvvar[c];
 	m_co_variance[c]=tempvar[c];   
       }
-    if(imagetype==1)
+    if(imagetype==2)
       for(c=1; c<noclasses+1; c++)
 	  m_mean(c,1)*=-1.0f;
   return 0;
@@ -1801,8 +1793,6 @@ float ZMRIMULTISegmentation::M_2PI(int numberofchan)
 {
   float val=1.0;
   for(int i=0;i<numberofchan;i++)
-    {
-      val=val*2*PI;
-    }
+    val=val*2*PI;
   return val;
 }
